%CLSMO Closed-loop state model.

%  Copyright
%  Charles L. Phillips  1994

delete clsm.dat
clc
clear ACL
diary clsm.dat
disp('                      CLOSED-LOOP STATE MODEL')
disp(' ')
diary off
disp(' ')
disp('    1.  A pole-assignment or steady-state LQ design is required.')
disp(' ')
disp('    2.  An estimator may be included, if designed.')
disp(' ')
estinc=0; % estimator not included in the closed-loop model

if poleas==1 | lqdes==1;

   s = 'n';
   if preest | curest | roeest | kalest ==1
      s=input(' Do you want the estimator included [y/n]? ','s')
   end

   if s=='y';

      estinc=1;

      if preest==1, % prediction estimator included

         ACL=A;
         AB=-B*K;
         BA=G*C;
         BB=A-G*C-B*K;
         order1=order+1;
         order2=2*order;
         ACL(order2,order2)=0;
         ACL(1:order,1:order)=A;
         ACL(1:order,order1:order2)=AB;
         ACL(order1:order2,1:order)=BA;
         ACL(order1:order2,order1:order2)=BB;

      elseif curest==1 | kalest==1, % current est or Kalman included

         AA=A;
         AB=-B*K;
         BA=G*C*A;
         BB=A-G*C*A-B*K;
         order1=order+1;
         order2=2*order;
         ACL(order2,order2)=0;
         ACL(1:order,1:order)=AA;
         ACL(1:order,order1:order2)=AB;
         ACL(order1:order2,1:order)=BA;
         ACL(order1:order2,order1:order2)=BB;

      elseif roeest==1, % reduced-order estimator included

         klen=length(K);
         for i=1:(klen-1)
            Kb(i)=K(i+1);
         end
         AA=A-K(1)*B*C;
         AB=-B*Kb;
         BA=G*C*A+(Aba-G*Aaa)*C-K(1)*(G*C*B+Bb-G*Ba)*C;
         BB=Abb-G*Aab-(G*C*B+Bb-G*Ba)*Kb;
         order1=order+1;
         order11=2*order-1;
         order2=2*order;
         ACL(order11,order11)=0;
         ACL(1:order,1:order)=AA;
         ACL(1:order,order1:order11)=AB;
         ACL(order1:order11,1:order)=BA;
         ACL(order1:order11,order1:order11)=BB;
    
      end, % if estimator included

   else, % no estimator included

      ACL = A - B*K;

   end,

   disp(' ')
diary on
   disp('  The closed loop system matrix ACL is:')
   disp(' ')
   ACL
   disp(' ')
diary off
   disp('  Press any key to continue ...'), pause, clc
   disp(' ')
diary on
   disp('  The characteristic equation of this matrix is: ')
   disp(' ')
   p=poly(ACL)
   disp(' ')
   disp('  The characteristic-equation roots are: ')
   disp(' ')
   r=roots(p)
diary off
   disp(' ')

else,  % No K design

   disp(' ')
diary on
   disp('  Neither pole assignment nor LQ design performed.')
   disp(' ')
diary off

end
disp(' ')
comp = computer;
if comp(1:2) ~= 'PC', clc, return, end
s=input(' Do you want the closed-loop model printed? [y/n]: ','s');
if s=='y'
   ! copy iden.dat prn:
   ! print clsm.dat
   clc
end

disp(' ')
disp('  Press any key to continue ...'), pause, clc


