%RES Reduced-order state estimator, discrete systems.

%  Copyright
%  Charles L. Phillips  1994

delete roes.dat
clc
clear G CC CCC AMC AMCI ale
%       Flags
preest=0; curest=0; roeest=1; kalest=0;
diary roes.dat
disp('                   REDUCED-ORDER ESTIMATOR DESIGN')
disp(' ')
diary off
disp(' ')
disp('  The plant is described by:')
disp(' ')
disp('                      x(k + 1) = Ax(k) + Bu(k)')
disp(' ')
disp('                          y(k) = Cx(k)')
disp(' ')
disp('  The state estimator is described by:')
disp(' ')
disp('                      q(k + 1) = Fq(k) + Gy(k) + Hy(k + 1) + Bu(k)')
disp(' '), 
disp('  where q(k) are the estimator states.  The order of the plant must') 
disp('  be at least two, and the plant C matrix must be of the form')
disp(' ')
disp('                             C = [1  0  0 ... 0]')
disp(' '); disp('  Press any key to continue ...'); pause; disp(' ')
disp('  The desired state-estimator characteristic equation is of the form,')
disp('  with n the system order and with ale(n - 1) = 1:')
disp(' ')
disp('    ale(n-1)*z^(n-1) + ale(n-2)*z^(n-2) + ... + ale(1)*z + ale(0) = 0')
disp(' ')
disp('  Enter the characteristic-equation coefficients in descending order')
disp('  of n, with each coefficient followed by pressing the enter key.')

for ii=1:order
   iii=order - ii;
   iiii=int2str(iii);
   str1=['    ale(',iiii,')= '];
   ale(ii) = input( str1);
end

disp(' ')
diary on
disp('  The characteristic-equation coefficients are:')
disp(' ')
ale
om1=order-1;

for L=1:om1
   Aba(L,1)=A(L+1,1);

   for M=1:om1
      Abb(L,M)=A(L+1,M+1);
      Aab(1,M)=A(1,M+1);
   end

end

Ba=B(1);

for nn=1:om1
   Bb(nn,1)=B((nn+1),1);
end

Aaa=A(1,1);
AMC=Aab;
AMCT=AMC;

for n=2:om1
   AMCT=AMCT*Abb;

   for nn=1:om1
      AMC(n,nn)=AMCT(1,nn);
   end

end

AMCI=inv(AMC);
CC=polyvalm(ale,Abb);
CCC = CC*AMCI;
disp(' ')
disp('  The gain matrix G is:')
disp(' ')
G=CCC(:,om1)
disp(' ')
diary off
disp('  Press any key to continue ...'), pause
diary on
disp(' ')
disp('  The closed-loop estimator matrix AG = (Abb - G*Aab) is:')
disp(' ')
AG=Abb-G*Aab
disp(' ')
disp('  For verification, the closed-loop characteristic equation ')
disp('  coefficients for the estimator, calculated from AG, are:')
disp(' ')
ale=poly(AG)
disp(' ')
disp('  The estimator characteristic-equation roots in rectangular form')
disp('  are:')
disp(' ')
r=roots(ale)
disp(' ')
disp('  The estimator characteristic-equation roots in polar form are:')
disp(' ')
magr=abs(r); angler=angle(r)*180/pi;
[magr,angler]
diary off
disp(' ')
comp = computer;
if comp(1:2) ~= 'PC', clc, return, end
s=input(' Do you want the results printed? [y/n]: ','s');
if s=='y'
   ! copy iden.dat prn:
   ! print roes.dat
   clc
end

disp(' ')
disp('  Press any key to return to the menu.'), pause, clc


